package com.kernelmaft.zanbur

import android.os.*
import androidx.activity.compose.*
import androidx.compose.foundation.layout.*
import androidx.compose.runtime.*
import androidx.compose.ui.*
import androidx.compose.ui.unit.*
import androidx.datastore.preferences.*
import androidx.datastore.preferences.core.*
import androidx.lifecycle.*
import kotlinx.coroutines.*
import kotlinx.coroutines.Dispatchers.IO
import kotlinx.coroutines.flow.*



class MainActivity : EdgeToEdgeActivity () {
	val dataStore = PreferenceDataStoreFactory . create { preferencesDataStoreFile ("state") }

	override fun onCreate ( savedInstanceState : Bundle ? ) {
		super . onCreate (savedInstanceState)

		Config . groups . forEach { AppState . addGroup (it) }

		val groups = mutableStateOf ( AppState . groups )
		AppState . subscribe { groups . value = it }

		lifecycleScope . launch (IO) {
			val prefs = dataStore . data . firstOrNull ()
			val savedSceneName = prefs ?. get ( stringPreferencesKey ("scene") )
			if ( savedSceneName != null ) {
				val savedScene = AppState . groups [0] . scenes . find { it . name == savedSceneName }
				savedScene ?. let { AppState . setCurrentScene ( 0 , it ) }
			}
		}

		setContent {
			AppFrame {
				Column ( Modifier . width ( 300 . dp ) ) {
					groups . value . forEach { group ->
						SceneSwitcher (group) { newScene ->
							AppState . setCurrentScene ( group . id , newScene )
							publishSceneChange ( group , newScene )
						}
					}
				}
			}
		}

		MqttClient . run (lifecycleScope)
	}

	override fun onStop () {
		super . onStop ()

		val currentScene = AppState . groups [0] . currentScene
		if ( currentScene != null ) lifecycleScope . launch (IO) {
			dataStore . edit {
				it [ stringPreferencesKey ("scene") ] = currentScene . name
			}
		}
	}
}
